#version 150

uniform sampler2D sampler0;
uniform vec2 samplerSize0;

#ifdef AlphaTest
uniform float alphaTestThreshold;    
#endif  
	
in vec4 color;
in vec4 texcoord0;

out vec4 fragColor;

#ifdef Fog
uniform float zNear;
uniform float zFar;

uniform float fogNear;
uniform float fogFar;
uniform vec4 fogColor;

float transformDepth(float z_b)
{
	// conversion into NDC [-1,1]
	z_b = z_b * 2.0 - 1.0;
	// cozNearversiozNear izNearto eye space
	return (2.0 * zNear * zFar / (zFar+zNear - z_b * (zFar - zNear)));
}
#endif

void main() {
	//TODO a unified read(x, y) function 
	#ifdef DisabledFilter
	//Base color to compute grayscale
    vec4 tmp = color * texture2DProj(sampler0, texcoord0);
    //Texel units in x and y direction
    float unx = 1.0/samplerSize0.x;
    float uny = 1.0/samplerSize0.y;
    //Find four neighbour texel alpha in order to create bevel effect
    //If coordinates are out of range, clamp alpha to zero
    vec2 nCoord = texcoord0.xy + vec2( 0.0,-uny);
    float na = (color * texture2D(sampler0, nCoord)).a * float( nCoord.y >= 0.0 );
    vec2 wCoord = texcoord0.xy + vec2(-unx, 0.0);
    float wa = (color * texture2D(sampler0, wCoord)).a * float( wCoord.x >= 0.0 );
    vec2 eCoord = texcoord0.xy + vec2( unx, 0.0);
    float ea = (color * texture2D(sampler0, eCoord)).a * float( eCoord.x < 1.0 );
    vec2 sCoord = texcoord0.xy + vec2( 0.0, uny);
    float sa = (color * texture2D(sampler0, sCoord)).a * float( sCoord.y < 1.0 );
    
    //Use YUV weights to compute grayscale
    float gray = dot(tmp, vec4(0.30, 0.59, 0.11, 0.00));
    //Bevel and clamp to normalized range
    gray = clamp(gray + na + wa - ea - sa, 0.0, 1.0);
    //Brighten, affecting dark values the most
    gray = (1 + gray)/2.0;
    //Emit grayscale color and original alpha(bevel is inside shape)
    fragColor = vec4(gray, gray, gray, tmp.a);
	#else
    fragColor = color * texture2DProj(sampler0, texcoord0);
    #endif
    
    #ifdef Fog
    float fogFactor = clamp((transformDepth(gl_FragCoord.z)-fogNear)/(fogFar-fogNear), 0.0, 1.0);
	fragColor.rgb = mix(fragColor.rgb, fogColor.rgb, fogFactor);
	fragColor.a = fragColor.a * mix(1.0, fogColor.a, fogFactor);
    #endif
    
    #ifdef Stipple
	fragColor *= vec4(float(mod(gl_FragCoord.x, 4.0)<2.0 ^^ mod(gl_FragCoord.y, 4.0)<2.0));
	#endif
	
    #ifdef AlphaTest
    if(fragColor.a < alphaTestThreshold)
    	discard;
	#endif 
}